#
#   board.py
#
#   This file is part of gshogi   
#
#   gshogi is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   gshogi is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with gshogi.  If not, see <http://www.gnu.org/licenses/>.
#   

import gtk
import os.path
import pieces

import engine
import gobject, time
from constants import *

class Board:


    def __init__(self):
        self.getboard() 
        self.wcap = engine.getcaptured(WHITE)
        self.bcap = engine.getcaptured(BLACK)       
        self.pieces = pieces.Pieces()        


    def build_board(self):
        self.myimage = [ \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()], \
            [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()]  \
            ]       


        self.wcap_image = [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()]
        self.wcap_label = [gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label()]
        self.bcap_image = [gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image(), gtk.Image()]
        self.bcap_label = [gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label(), gtk.Label()]        

        self.piece_pixbuf = []        

        self.init_board()        


    def set_refs(self, game, gui):
        self.game = game
        self.gui = gui      


    def init_board(self):
        prefix = self.game.get_prefix()

        self.pieces.load_pieces(prefix)  

        # pieces captured by black (index 0) and white (index 1)
        self.bcap2 = [ ['0', '0', '0', '0', '0', '0', '0', '0', '0'],
                       ['0', '0', '0', '0', '0', '0', '0', '0', '0'] ] 
            
    
        # pieces contains the list of possible pieces in self.board_position
        pieces = [" -", " p", " l", " n", " s", " g", " b", " r", " k", "+p", "+l", "+n", "+s", "+b", "+r", " P", " L", " N", " S", " G", " B", " R", " K", "+P", "+L", "+N", "+S", "+B", "+R"]           

        #
        # loop through the board squares and set the pieces
        # x, y = 0, 0 is the top left square of the board
        #
        for x in range(0, 9):
            for y in range(0, 9):
                # convert the x, y square to the location value used by the engine
                l = self.get_gs_loc(x, y)

                # get the piece at (x, y)
                piece = self.board_position[l]
            
                # set the image on the board square to the required piece
                self.myimage[x][y].set_from_pixbuf(self.pieces.getpixbuf(piece))               
                
                # call gui to show this square
                self.gui.init_board_square(self.myimage[x][y], x, y)               

        # initialise white komadai (area containing pieces captured by white)        
        for y in range(0, 9):
            self.wcap_image[y].set_from_pixbuf(self.pieces.getpixbuf(" -"))
            self.wcap_label[y].set_text("   ")            
            # call gui to show this square
            self.gui.init_wcap_square(self.wcap_image[y], y, self.wcap_label[y])
            

        # initialise black komadai (area containing pieces captured by black)        
        for y in range(0, 9):
            self.bcap_image[y].set_from_pixbuf(self.pieces.getpixbuf(" -"))
            self.bcap_label[y].set_text("   ") 
            # call gui to show this square
            self.gui.init_bcap_square(self.bcap_image[y], y, self.bcap_label[y])        
        
        self.gui.set_window_size()        
        gobject.idle_add(self.update)

    def get_gs_loc(self, x, y):        
        l = x + (8 - y) * 9
        return l


    # convert gshogi co-ordinates for square into standard notation (e.g.  (2, 6) -> 7g)
    def get_square_posn(self, x, y):
        lets = 'abcdefghi'
        sq = str(9 - x) + lets[y]
        return sq      


    # convert standard notation for square into gshogi co-ordinates (e.g.  7g -> (2, 6) )
    def get_gs_square_posn(self, sq):
        x = 9 - int(sq[0:1])
        lets = 'abcdefghi'
        y = lets.index(sq[1:2])        
        return x, y      
    

    def display_board(self):        

        sf = self.get_scale_factor()        
        self.pieces.set_scale_factor(sf)        

        #
        # loop through the board squares and set the pieces
        # x, y = 0, 0 is the top left square of the board
        #
        for x in range(0, 9):
            for y in range(0, 9):
                # convert the x, y square to the location value used by the engine
                l = self.get_gs_loc(x, y)  
                piece = self.board_position[l]                
                self.myimage[x][y].set_from_pixbuf(self.pieces.getpixbuf(piece))


    def display_capturedw(self, wcap):   

        #
        # wcap contains a list of captured pieces  e.g.
        # 0P 0L 0N 0S 0G 0B 0R 0P 0L 0N 0S 0B 0R 0K 
        #            

        self.bcap2[WHITE] = ['0', '0', '0', '0', '0', '0', '0', '0', '0']

        for i in range(0, 9):
            self.wcap_image[i].set_from_pixbuf(self.pieces.getpixbuf(" -"))
            self.wcap_label[i].set_text("   ")         

        i = 0
        for c in wcap:
            if c == '':
                continue            
            piece = c[1:2]
            num = c[0:1]            
                     
            if i < 9:
                z = " " + piece                
                self.wcap_image[i].set_from_pixbuf(self.pieces.getpixbuf(z))
                self.bcap2[WHITE][i] = piece                
                self.wcap_label[i].set_text(" " + num + " ")    

            i = i + 1


    def display_capturedb(self, bcap):        

        self.bcap2[BLACK] = ['0', '0', '0', '0', '0', '0', '0', '0', '0']

        for i in range(0, 9):            
            self.bcap_image[i].set_from_pixbuf(self.pieces.getpixbuf(" -"))
            self.bcap_label[i].set_text("   ")   

        i = 8
        for c in bcap:
            if c == '':
                continue
            piece = c[1:2]
            num = c[0:1]            
                      
            if i > 0:                
                z = " " + piece
                z = z.lower()
                self.bcap_image[i].set_from_pixbuf(self.pieces.getpixbuf(z))
                self.bcap2[BLACK][i] = piece
                self.bcap_label[i].set_text(" " + num + " ")  
            i = i - 1


    #
    # test if user has clicked on a valid source square
    # i.e. one that contains a black piece if side to move is black
    #      otherwise white piece
    #
    def valid_source_square(self, x, y, stm):
        l = self.get_gs_loc(x, y)      
        piece = self.board_position[l]
        
        pieces = [ [" p", " l", " n", " s", " g", " b", " r", " k", "+p", "+l", "+n", "+s", "+b", "+r"],
                   [" P", " L", " N", " S", " G", " B", " R", " K", "+P", "+L", "+N", "+S", "+B", "+R"] ]
                 
 
        # black_pieces = [" p", " l", " n", " s", " g", " b", " r", " k", "+p", "+l", "+n", "+s", "+b", "+r"]        
        try:
            idx = pieces[stm].index(piece)
        except ValueError, ve:
            return False

        return True


#
# Check if promotion is mandatory (i.e on last rank of board for 
# pawn/lance or last 2 ranks for knight) or optional
# 
# return
# 0 - no promotion
# 1 - promotion optional
# 2 - promotion mandatory
#
    def promote(self, src_x, src_y ,dst_x ,dst_y, stm):
        l = self.get_gs_loc(src_x, src_y)  
        piece = self.board_position[l]

        # check for mandatory
        if stm == BLACK:
            if (dst_y == 0):        
                if (piece == " p" or piece == " l" or piece == " n"):
                    return 2
            elif (dst_y == 1): 
                if (piece == " n"):
                    return 2
        else:
            if (dst_y == 8):        
                if (piece == " P" or piece == " L" or piece == " N"):
                    return 2
            elif (dst_y == 7): 
                if (piece == " N"):
                    return 2


        
        # check for optional        
        #        BlackPawn,    BlackLance,   BlackKnight, BlackSilver,
        #        BlackBishop,  BlackRook,    
        promotable_pieces = [ [" p", " l", " n", " s", " b", " r"],
                              [" P", " L", " N", " S", " B", " R"] ] 

        try:
            idx = promotable_pieces[stm].index(piece)
        except ValueError, ve:
            return 0

        return 1


    def use_western_pieces(self, wps):
        self.pieces.use_western_pieces(wps)        
        self.refresh_screen()


    def getboard(self):
        self.board_position = engine.getboard()        


    # work out a factor to scale the pieces by
    # this changes when the user resizes the board
    def get_scale_factor(self):        
        square_size = self.gui.get_square_size()
        width = square_size.width
        height = square_size.height
        if width < 15:
            factor = 1.0
        else:
            factor = width / 64.0         
        
        return factor


    def update(self):
        self.getboard()         
        self.wcap = engine.getcaptured(WHITE)
        self.bcap = engine.getcaptured(BLACK)        
        #print "wcap=",self.wcap
        #print "bcap=",self.bcap
        self.refresh_screen()


    def refresh_screen(self):
        self.display_board()
        self.display_capturedw(self.wcap)        
        self.display_capturedb(self.bcap)        


    def get_cap_piece(self, y, stm):        
        return self.bcap2[stm][y]
       
 
    """
    # routine to update the board after a move when using a USI engine
    #def usi_do_move(self, src_x, src_y, dst_x, dst_y, move):  
    def usi_do_move(self, move, side_to_move):        

        #print "usi_do_move,move=",move,"side_to_move=",side_to_move     
        # set source square to unoccupied
        if move[1:2] == '*':
            if side_to_move == BLACK:
                cap = self.bcap
            else:
                cap = self.wcap
            
            found = False
            j = -1
            for c in cap:
                j += 1
                if c == '':
                    continue
                piece = c[1:2]
                num = c[0:1]
                
                if piece == move[0:1]:
                    n = int(num)
                    if n < 1:
                        print "error2 in usi_do_move, move=", move, "side to move=",side_to_move
                    if n == 1:
                        cap[j] = ''
                    else:                        
                        n -= 1                        
                        p = str(n) + piece                
                        cap[j] = p
                    if side_to_move == BLACK:
                        piece = piece.lower()
                    else:
                        piece = piece.upper()
                    piece = ' ' + piece
                    found = True
                    break
            if not found:
                print "error1 in usi_do_move, move=", move, "side to move=",side_to_move
        else:
            src_x, src_y = self.get_gs_square_posn(move[0:2])
            #print "source x, y=",src_x, src_y
            l = self.get_gs_loc(src_x, src_y) 
            piece = self.board_position[l]
            self.board_position[l] = ' -'

        # set destination square to moved piece
        dst_x, dst_y = self.get_gs_square_posn(move[2:4])
        #print "destn x, y=",dst_x, dst_y
        l = self.get_gs_loc(dst_x, dst_y)
        dest_piece = self.board_position[l]
        if move.endswith('+'):
            piece = '+' + piece[1:2]
            #print "promo val=",piece
        #print "setting piece to",piece
        self.board_position[l] = piece
        
        # if piece was captured add it to the komadai 
        if dest_piece != ' -':
            if side_to_move == BLACK: 
                cap = self.bcap
            else:
                cap = self.wcap
    
            i = 8
            j = -1
            found = False
            for c in cap:
                j += 1
                if c == '':
                    continue
                piece = c[1:2]
                num = c[0:1]

                dp = dest_piece[1:2]
                if piece == dp.upper():
                    n = int(num)
                    n += 1                    
                    p = str(n) + piece                
                    cap[j] = p
                    found = True
                    break

            if not found:
                j = -1
                for c in cap:
                    j += 1
                    if c == '':
                        dp = dest_piece[1:2]
                        cap[j] = '1' + dp.upper()                           
                        break


        #print "updated board=",self.board_position
        #print "bcap=",self.bcap
        #print "wcap=",self.wcap
    """


