#!/usr/bin/env python

#
#   gshogi 0.3  October 2010
#
#   Copyright (C) 2010 John Cheetham    
#   
#   web   : http://www.johncheetham.com/projects/gshogi
#   email : developer@johncheetham.com
#   
#   This file is part of gshogi
#
#   gshogi is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   gshogi is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with gshogi.  If not, see <http://www.gnu.org/licenses/>.
#   

import pygtk
pygtk.require('2.0')
import gtk

import thread, traceback
import sys
import os
import engine
import usi
import gobject
import pickle
import time

import gui, board, engine_manager
from constants import *

class Game:    

    def __init__(self):        
        
        self.ask_before_promoting = False
        self.gameover = False       
        self.time_limit = '00:10'
        self.stopped = True        
        self.src = ''
        self.src_x = ''
        self.src_y = ''
        
        # Time Control
        self.tc_hours = 0
        self.tc_mins = 10
        self.tc_byoyomi = 10
                
        time_ms = (self.tc_hours * 60 * 60 + self.tc_mins * 60) * 1000
        # btime, wtime ,byoyomi times in milliseconds
        self.time_left = [time_ms, time_ms, self.tc_byoyomi * 1000]         

        self.search_depth = 39
        self.thinking = False
        self.cmove = 'none'
        self.movelist = []
        self.redolist = []        
        self.player = ["Human", "gshogi"]                    

        # usiw is the instance that plays white (gote)
        # usib is the instance that plays black (sente)
        self.usib = usi.Usi()
        self.usiw = usi.Usi()       

        # set paths to images. opening book etc        
        self.set_data_paths() 
        opening_book_path = os.path.join(self.prefix, "data/opening.bbk")              
        engine.init(opening_book_path)        

        # instantiate board, gui, classes 
        self.engine_manager = engine_manager.Engine_Manager()         
        self.board = board.Board()
        self.gui = gui.Gui() 
        self.board.set_refs(self, self.gui)
        self.gui.set_refs(self, self.board, self.engine_manager)
        self.gui.build_gui()
        self.board.build_board()
        self.engine_manager.set_refs(self, self.gui, self.usib, self.usiw)
        self.usib.set_refs(self, self.engine_manager, self.gui)
        self.usiw.set_refs(self, self.engine_manager, self.gui)        

        # set level        
        command = 'level 0 ' + self.time_limit                  
        engine.command(command)   
        # turn off beeps
        if not BEEP:
            engine.command('beep')

        # restore users settings to values from previous game
        self.restore_settings()

        self.usib.set_engine(self.player[BLACK], None)
        self.usiw.set_engine(self.player[WHITE], None)        
        self.gui.update_toolbar(self.player)
        self.gui.set_toolbar_time_control(self.time_left, 0, BLACK)
        self.gui.set_toolbar_time_control(self.time_left, 0, WHITE)         
        
        self.gui.enable_go_button()            
        self.gui.disable_stop_button()

        self.stm = self.get_side_to_move()
        self.clock_start_time = time.time()
        self.clock_stm = self.stm
        self.timer_active = False        
        

    def set_data_paths(self):
        
        # Find the absolute path that this python program is running in 
        progpath = os.path.abspath(os.path.dirname(__file__))        

        # work out if we are running from an installed version
        # or from the source directory
        if progpath.startswith(sys.prefix):
            # we are installed
            self.prefix = os.path.join (sys.prefix, "share/gshogi")
            
            if os.path.isdir(self.prefix):
                print "images/data path=", self.prefix
            else:
                print "setting images/data path"

                for dir in ("share", "games", "share/games",
                    "local/share", "local/games", "local/share/games"):
                    self.prefix = os.path.join (sys.prefix, dir, "gshogi")
                    if os.path.isdir(self.prefix):
                        print "found images/data path=", self.prefix                        
                        break
                else:
                    raise Exception("can't find data directory")
       
        else:
            # we are NOT installed
            # get data files (images, opening book, endgame databases) from same directory as this program 
            self.prefix = os.path.abspath(os.path.dirname(__file__))
            self.prefix = os.path.dirname(self.prefix)            
            print "using images/data path=", self.prefix 


        # set up gshogi directory under home directory
        self.gshogipath = os.path.expanduser("~") + "/.gshogi"        
        if not os.path.exists(self.gshogipath):
            try:
                os.makedirs(self.gshogipath)
            except OSError, exc:                
                raise


    #
    # Process Human move
    #
    def square_clicked(self, widget, event, data):               
        if self.gameover or self.thinking or self.stopped:        
            return

        self.stm = self.get_side_to_move()
        if self.player[self.stm]  != "Human":
            return        
        self.gui.set_side_to_move(self.stm)

        # get x,y co-ords of square clicked on (0, 0 is top left)            
        (x, y) = data        

        # convert the x, y co-ords into the shogi representation (e.g. 8, 6 is 1g)
        sq = self.board.get_square_posn(x, y)        
        
        # if the square clicked on is a valid source square
        # then set this square as the source square
        if self.board.valid_source_square(x, y, self.stm): 
            print "--------------------------------------------------------------------------------"
            #print "blacks move"
            print self.get_side_to_move_string(self.stm) + "s move"               
            self.src = sq
            #print "source square: (x, y) = (", x, ",",  y, ") ", sq
            self.src_x = x
            self.src_y = y
            return
          
        # must have a valid source square before checking dest square        
        if self.src == '':
            return
             
        # Not a valid source square, assume destination square            
        self.dst = sq
        move = self.src + self.dst
            
        # check for promotion
        if self.promotion_zone(self.src, self.dst, self.stm):
            promote = self.board.promote(self.src_x, self.src_y, x, y, self.stm)                           
            if (promote == 2): 
                # must promote                    
                move = move + "+" 
            elif (promote == 1):                    
                # promotion is optional             
                if self.ask_before_promoting:                        
                    response = self.gui.promote_popup()
                    if (response == gtk.RESPONSE_CANCEL):
                        return
                    if (response == gtk.RESPONSE_YES):                            
                        move = move + "+"
                else:                        
                    move = move + "+"           

        print "move=",move        

        engine.setplayer(self.stm)
        validmove = engine.hmove(move)
        if (not validmove):
            # illegal move                               
            self.gui.set_status_bar_msg("Illegal Move")
            return                         
        
        self.movelist.append(move)
        self.redolist = []
        
        self.board.update()                
        engine.command('bd')
        self.src = ''

        self.gameover, msg = self.check_for_gameover() 
        if (self.gameover):
            self.stop()               
            self.gui.set_status_bar_msg(msg)                
            self.thinking = False  
            return                   
                
        #self.gui.set_status_bar_msg('Thinking ...')  
        print "--------------------------------------------------------------------------------"
        #print "whites move"
        self.stm = self.get_side_to_move()
        self.gui.set_side_to_move(self.stm)
        print self.get_side_to_move_string(self.stm) + "s move"
        self.gui.set_status_bar_msg(" ")                
                
        self.src = ''

        if self.player[self.stm] == "Human":                

            # update time for last move                     
            elapsed_time = (time.time() - self.clock_start_time) * 1000 
            self.time_left[self.clock_stm] = int(self.time_left[self.clock_stm] - elapsed_time)
            if self.time_left[self.clock_stm] < 0:
                self.time_left[self.clock_stm] = 0                
            self.gui.set_toolbar_time_control(self.time_left, 0, self.clock_stm)                

            # set clock ready for move to come   
            self.clock_stm = self.stm
            self.clock_start_time = time.time()

            if not self.timer_active:
                gobject.timeout_add(1000, self.show_time)

            return
            
        self.thinking = True
        # disable some functionality while the computer is thinking
        #self.gui.disable_menu_items()   

        # It's the computers turn to move
        # kick off a separate thread for computers move so that gui is still useable                                      
        self.ct= thread.start_new_thread( self.computer_move, () )
                                        
        return
        

    def stop_clicked(self, widget):
        self.stop()


    def stop(self):
        self.stopped = True        
        self.gui.enable_menu_items()
        self.gui.enable_go_button()
        self.gui.disable_stop_button()

        # update time
        elapsed_time = (time.time() - self.clock_start_time) * 1000
        self.time_left[self.clock_stm] = int(self.time_left[self.clock_stm] - elapsed_time)
        if self.time_left[self.clock_stm] < 0:
            self.time_left[self.clock_stm] = 0
        self.gui.set_toolbar_time_control(self.time_left, 0, self.clock_stm)                 

        # stop engines        
        #self.gui.set_status_bar_msg("stopping")
        self.usib.stop_engine()
        self.usiw.stop_engine()
        engine.movenow()
        self.gui.set_status_bar_msg("stopped")        


    def go_clicked(self, widget):        
            
        # update time control prior to move            
        self.gui.set_toolbar_time_control(self.time_left, 0, self.stm)            
            
        # side to move
        self.stm = self.get_side_to_move() 

        # start a timer to display the time left while the player is thinking                    
        self.clock_start_time = time.time()        
        self.clock_stm = self.stm
        if not self.timer_active:       
            gobject.timeout_add(1000, self.show_time)

        self.gui.disable_menu_items()
        self.gui.disable_go_button()
        self.gui.enable_stop_button()        
        
        self.stopped = False               

        if self.player[self.stm] == "Human":
            self.gui.set_status_bar_msg("ready")             
            return

        self.gui.set_status_bar_msg('Thinking ...')
        # It's the computers turn to move
        # kick off a separate thread for computers move so that gui is still useable                                                    
        self.ct= thread.start_new_thread( self.computer_move, () )       


    def cap_square_clicked(self, widget, event, data):        
        if self.gameover or self.thinking:        
            return            
        
        x, y, colour = data

        # If user clicked in the black capture area and it's white to move or vice versa
        # then ignore
        stm = self.get_side_to_move()
        if stm != colour:
            return

        self.src_x = x
        self.src_y = y                
        piece = self.board.get_cap_piece(y, stm)        

        if (piece != '0'):
            self.src = piece + '*'
        else:
            self.src = ''
        #print "self.src=",self.src


    def get_prefix(self):
        return self.prefix


    def computer_move(self):        
        try:           
            self.thinking = True                
                       
            while  True:
                self.thinking = True                                
                self.stm = self.get_side_to_move()                                

                # update time for last move                     
                elapsed_time = (time.time() - self.clock_start_time) * 1000 
                self.time_left[self.clock_stm] = int(self.time_left[self.clock_stm] - elapsed_time)
                if self.time_left[self.clock_stm] < 0:
                    self.time_left[self.clock_stm] = 0
                gtk.gdk.threads_enter()
                self.gui.set_toolbar_time_control(self.time_left, 0, self.clock_stm)
                self.gui.set_side_to_move(self.stm)
                gtk.gdk.threads_leave()

                # set clock ready for move to come   
                self.clock_stm = self.stm
                self.clock_start_time = time.time()

                if self.player[self.stm] == "Human":
                    self.thinking = False                              
                    return
                
                t_start = time.time()
                if self.player[self.stm] != 'gshogi':                            
                    if self.stm == BLACK:
                        self.usi = self.usib
                    else:
                        self.usi = self.usiw
                    
                    self.cmove = self.usi.cmove(self.movelist, self.time_left, self.stm)                    
                    if self.stopped or self.cmove is None:                        
                        self.thinking = False                        
                        return                    

                    print "computer move is",self.cmove
                    # check if player resigned
                    if self.cmove == 'resign':
                        print "computer resigned"                        
                        self.gameover = True
                        self.thinking = False
                        colour = self.get_side_to_move_string(self.stm)                        
                        msg = "game over - " + colour + " resigned"
                        gtk.gdk.threads_enter()
                        self.stop()
                        self.gui.set_status_bar_msg(msg)                        
                        gtk.gdk.threads_leave()                        
                        self.thinking = False 
                        return
                    
                    #engine.setplayer(WHITE)
                    engine.setplayer(self.stm)            
                    validmove = engine.hmove(self.cmove)             
                    if (not validmove):
                        gtk.gdk.threads_enter()
                        self.stop()
                        self.gui.set_status_bar_msg(self.cmove + " - computer made illegal Move!")                        
                        gtk.gdk.threads_leave()
                        self.gameover = True                        
                        self.thinking = False                 
                        return                    
                    engine.command('bd')                                       
                else:

                    #
                    # We are using the builtin gshogi engine (not a USI engine)
                    #

                    if self.player[self.stm ^ 1] == "Human":
                        gtk.gdk.threads_enter()                    
                        self.gui.set_status_bar_msg('Thinking ...')
                        gtk.gdk.threads_leave() 

                    # set the computer to black or white                                       
                    engine.setplayer(self.stm ^ 1)
                    
                    # set time limit for move in gshogi engine
                    gs_movetime = self.time_left[self.stm]
                    gs_byoyomi = self.time_left[2]
                    if gs_movetime > 0:
                        gs_movetime = gs_movetime / 80
                    elif gs_byoyomi > 0:
                        gs_movetime = gs_byoyomi
                    else:
                        # no time left - set to 1 second
                        gs_movetime = 1000

                    # if less than 1 second set to 1 second
                    if gs_movetime < 1000:
                        gs_movetime = 1000
                    
                    gs_secs = gs_movetime / 1000
                    gs_mins = gs_secs / 60;
                    if gs_mins > 59:                    
                        gs_mins = 59
                        gs_secs = 59                    
                    elif gs_mins > 0:                    
                        gs_secs = gs_secs % (gs_mins * 60)
                    
                    gs_mins = int(gs_mins)
                    gs_secs = int(gs_secs)

                    gs_mins_str = str(gs_mins)
                    gs_secs_str = str(gs_secs)
                    if gs_mins < 10:
                        gs_mins_str = '0' + gs_mins_str
                    if gs_secs < 10:
                        gs_secs_str = '0' + gs_secs_str

                    gs_level = gs_mins_str + ':' + gs_secs_str                    
                    
                    command = 'level 0 ' + gs_level
                    engine.command(command)                                  

                    # call the gshogi engine to do the move      
                    self.cmove = engine.cmove()                                                         

                    if self.stopped:                        
                        self.thinking = False
                        gtk.gdk.threads_enter()
                        self.gui.set_status_bar_msg("stopped")                        
                        gtk.gdk.threads_leave()
                        engine.command('undo')                        
                        return                                       
                           
                if self.cmove != '':
                    self.movelist.append(self.cmove)
                    self.redolist = []
                else:
                    # empty move is returned by gshogi engine when it is in checkmate
                    print "empty move returned by engine"                         

                # if the engine moved very fast then wait a bit
                # before displaying the move. The time to wait
                # is in MIN_MOVETIME in constants.py
                t_end = time.time()
                move_t = t_end - t_start
                if move_t < MIN_MOVETIME:
                    diff = MIN_MOVETIME - move_t
                    time.sleep(diff)

                # show computer move
                gtk.gdk.threads_enter()                
                self.board.update()
                gtk.gdk.threads_leave()    
    
                engine.command('bd')
       
                print "move=",self.cmove
                msg = self.cmove                

                self.gameover, gmsg = self.check_for_gameover()
                if (self.gameover):            
                    if (msg == ''):
                        msg = gmsg 
                    else:
                        msg = self.get_side_to_move_string(self.stm) + ": " + msg
                        msg = msg + ". " + gmsg                                   
                    self.thinking = False
                    self.stm = self.get_side_to_move()
                    gtk.gdk.threads_enter()
                    self.stop()                
                    self.gui.set_side_to_move(self.stm)                    
                    self.gui.set_status_bar_msg(msg)                    
                    gtk.gdk.threads_leave()                 
                    return    

                msg = self.get_side_to_move_string(self.stm) + ": " + msg
                gtk.gdk.threads_enter()
                self.gui.set_status_bar_msg(msg)
                gtk.gdk.threads_leave() 

            self.thinking = False 
        except:            
            traceback.print_exc()
            return

    # timer to countdown the clock on the screen during a move
    def show_time(self):        
        self.timer_active = True        
        if self.stopped:
            self.timer_active = False            
            return False
        elapsed_time = int((time.time() - self.clock_start_time) * 1000)      

        self.gui.set_toolbar_time_control(self.time_left, elapsed_time, self.clock_stm)             
        
        return True


    def check_for_gameover(self):
        gameover = False
        msg = ''
        winner = engine.getwinner()                
        if (winner):
            gameover = True            
            winner -= 1
            if (winner == BLACK):
                msg = "game over - black wins"
            elif (winner == WHITE):
                msg = "game over - white wins"
            elif (winner == NEUTRAL):
                msg = "game over - match drawn"
            else:
                print "invalid value returned from engine getwinner function"
        return gameover, msg      


    def set_promotion_mode(self, mode):
        self.ask_before_promoting = mode               


    def promotion_zone(self, src, dst, stm):        
        srclet = src[1:2]                        
        dstlet = dst[1:2]

        # can't promote dropped pieces
        if srclet == '*':
            return False                

        if stm == BLACK:
            if (srclet < 'd' and srclet != '*') or dstlet < 'd':
                return True
            else:
                return False
        else:
            if (srclet > 'f' and srclet != '*') or dstlet > 'f':
                return True
            else:
                return False


    #
    # Callback Functions
    #
    
    def quit_game(self, b):
        self.quit()        
        return False

    
    # This callback quits the program
    def delete_event(self, widget, event, data=None):
        self.quit()        
        return False


    def quit(self): 
        self.stopped = True
        engine.movenow()
        self.save_settings()         
        self.usib.stop_engine() 
        self.usiw.stop_engine()      
        gtk.main_quit()
        return False


    def new_game(self, b):        
        self.gameover = False
        engine.command('new') 
        self.board.update()
        if not BEEP:
            engine.command('beep')        

        self.usib.set_newgame()
        self.usiw.set_newgame()        
        self.movelist = []
        self.redolist = [] 
        self.gui.set_status_bar_msg('') 
        self.stm = self.get_side_to_move()
        self.gui.set_side_to_move(self.stm)        
                                         
        time_ms = (self.tc_hours * 60 * 60 + self.tc_mins * 60) * 1000
        # btime, wtime ,byoyomi times in milliseconds
        self.time_left = [time_ms, time_ms, self.tc_byoyomi * 1000]

        self.gui.set_toolbar_time_control(self.time_left, 0, BLACK)
        self.gui.set_toolbar_time_control(self.time_left, 0, WHITE)      
    

    # Load game from a previously saved game    
    def load_game(self, b):        
        
        dialog = gtk.FileChooserDialog("Load..",
                               None,
                               gtk.FILE_CHOOSER_ACTION_OPEN,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_folder(os.path.expanduser("~"))

        filter = gtk.FileFilter()  
        filter.set_name("gshog files")      
        filter.add_pattern("*.gshog")        
        dialog.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)        

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            fname = dialog.get_filename()            
            rc = engine.loadgame(fname)
            if (rc > 0):
                self.gui.set_status_bar_msg("Error loading game")
            else:               
                f = open(fname)
                startmoves = False
                self.movelist = []
                self.redolist = []
                for line in f:
                    if startmoves:                        
                        l = line.strip()
                        sl = l.split()
                        m = sl[0]
                        if m.startswith('+'):
                            m = m[1:]
                        if m.find('*') != -1:        
                            move = m
                        else:                                
                            move = m[1:]                                
                        self.movelist.append(move)                            
                    if line.startswith('  move   score depth'):
                        startmoves = True
                f.close()               
                self.usib.set_newgame()
                self.usiw.set_newgame()
                self.gui.set_status_bar_msg("game loaded")
                self.gameover = False            
            self.board.update()
            self.stm = self.get_side_to_move()
            self.gui.set_side_to_move(self.stm)                
            
            time_ms = (self.tc_hours * 60 * 60 + self.tc_mins * 60) * 1000
            # btime, wtime ,byoyomi times in milliseconds
            self.time_left = [time_ms, time_ms, self.tc_byoyomi * 1000] 

            self.gui.set_toolbar_time_control(self.time_left, 0, BLACK)
            self.gui.set_toolbar_time_control(self.time_left, 0, WHITE)                         
                    
        dialog.destroy()

         
    # get board from saved game
    def get_board(self, fname):
        startboard = False
        print "opening",fname
        f = open(fname)
        sfen = ''
        board_state = ''
        try:
            for i in range(0, 6):
                f.readline()  
            for i in range(0, 9):
                l = f.readline() 
                l = l.strip() 
                sl = l.split()                 
                rank = sl[1:10]                
                s = ''
                bc = 0
                for e in rank:
                    if e != '-':
                        if bc > 0:
                            s = s + str(bc)
                            bc = 0                        
                        s = s + e.swapcase()
                    else:
                        bc += 1
                if bc > 0:
                    s = s + str(bc)
                if i != 8:                    
                    s = s + '/'                 
                board_state = board_state + s

            sfen = sfen + board_state

            # side to move
            sfen = sfen + ' b'

            for i in range(0, 4):
                f.readline()  

            # black pieces in hand            
            ihb = self.pieces_in_hand(f.readline())            

            # white pieces in hand
            ihw = self.pieces_in_hand(f.readline())            

            ih = ihb + ihw
            if ih == '':
                ih = '-'

            sfen = sfen + ' ' + ih                                      

        except:
            print "error reading file in get_board"
       
        f.close()
        
        return sfen


    def pieces_in_hand(self, l):
        print "l=",l
        l = l.strip() 
        sl = l.split()
        side = sl.pop(0)
        ih = ''
        ps = ['p', 'l', 'n', 's', 'g', 'b', 'r', 'k']
        i = 0        
        for p in ps:
            if side == 'B':
                p = p.upper()
            c = int(sl[i])            
            if c > 0:
                if c == 1:                        
                    ih = ih + p
                else:
                    ih = ih + str(c) + p
            i += 1
       
        return ih


    # Save game to a file   
    def save_game(self, b):        

        dialog = gtk.FileChooserDialog("Save..",
                               None,
                               gtk.FILE_CHOOSER_ACTION_SAVE,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_folder(os.path.expanduser("~"))

        filter = gtk.FileFilter()  
        filter.set_name("gshog files")      
        filter.add_pattern("*.gshog")        
        dialog.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)        

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            if filename[-6:] != '.gshog':
                filename = filename + '.gshog' 
            engine.command('save ' + filename)      
                 
        dialog.destroy()


    #
    # save users settings at program termination
    #    
    def save_settings(self):         
        
        # get settings
        s = Settings()
        s.name = NAME
        s.version = VERSION
        s.engine_list = self.engine_manager.get_engine_list()
        s.western = self.gui.get_western()
        s.player_white = self.player[WHITE]
        s.player_black = self.player[BLACK]
        s.tchours = self.tc_hours
        s.tcminutes = self.tc_mins
        s.tcbyoyomi = self.tc_byoyomi          
            
        # pickle and save settings
        try:                        
            settings_file = os.path.join (self.gshogipath, "settings")            
            f = open(settings_file, 'w')            
            pickle.dump(s, f)            
            f.close()        
        except AttributeError, ae:
            print "attribute error:",ae
        except pickle.PickleError, pe:
            print "PickleError:", pe
        except pickle.PicklingError, pe2:
            print "PicklingError:", pe2
        except Exception, exc:
            print "cannot save settings:", exc         


    #
    # restore users settings at program start-up
    #    
    def restore_settings(self):       
        x = ''               
        try:            
            settings_file = os.path.join (self.gshogipath, "settings")            
            f = open(settings_file, 'rb')            
            x = pickle.load(f)           
            f.close()        
        except EOFError, eofe:
            print "eof error:",eofe        
        except pickle.PickleError, pe:
            print "pickle error:", pe
        except IOError, ioe:
            pass    # Normally this error means it is the 1st run and the settings file does not exist        
        except Exception, exc:
            print "Cannot restore settings:", exc             
        
        if x:
            # engine list
            try:                
                self.engine_manager.set_engine_list(x.engine_list)                
            except Exception, e:                        
                print "engine list not restored"                         

            # using eastern or western pieces
            try:
                if x.western == True:
                    self.gui.set_western()
            except Exception, e:                        
                print "pieces setting not restored" 

            # set the engine or human for each player
            try:
                self.player[WHITE] = x.player_white
                self.player[BLACK] = x.player_black
            except Exception, e:                        
                print "player setting not restored" 

            # time controls
            try:            
                self.tc_hours = x.tchours
                self.tc_mins = x.tcminutes
                self.tc_byoyomi = x.tcbyoyomi       

                time_ms = (self.tc_hours * 60 * 60 + self.tc_mins * 60) * 1000
                # btime, wtime ,byoyomi times in milliseconds
                self.time_left = [time_ms, time_ms, self.tc_byoyomi * 1000]          
            except Exception, e:                        
                print "time controls not restored" 


    def undo_move(self, b):        
        engine.command('undo')
        move = None 
        try:            
            move = self.movelist.pop()                        
            self.redolist.append(move)           
            self.stm = self.get_side_to_move()
            self.gui.set_side_to_move(self.stm)            
        except IndexError:
            pass
        
        try:
            self.usib.stop_engine()
            self.usiw.stop_engine()     
        except:
            pass           
        self.gameover = False
        self.board.update()
        if move is not None:        
            self.gui.set_status_bar_msg('(' + move + ')')
        

    def redo_move(self, b):
        move = None        
        try:
            move = self.redolist.pop()

            # get side to move before appending to movelist            
            self.stm = self.get_side_to_move()
            self.movelist.append(move)

            # do the move in gshogi engine
            engine.setplayer(self.stm)            
            engine.hmove(move) 
            engine.command('bd')

            # side to move changes to opponent
            self.stm = self.get_side_to_move()
            self.gui.set_side_to_move(self.stm)            
        except IndexError:
            pass
        
        try:
            self.usib.stop_engine()
            self.usiw.stop_engine()     
        except:
            pass        
        self.board.update()        
        if move is not None:        
            self.gui.set_status_bar_msg(move)         


    def get_side_to_move(self):          
        if len(self.movelist) % 2 == 0:            
            return(BLACK)
        else:
            return(WHITE)
                

    def get_side_to_move_string(self, stm):          
        if stm == BLACK:            
            return "black"
        else:
            return "white"


    def get_player(self, side):
        return self.player[side]


    def move_now(self, b):
        if self.player[self.stm]  == "Human":
            return

        if not self.thinking:
            return

        # builtin gshogi engine
        if self.player[self.stm]  == "gshogi":
            engine.movenow()
            return
        
        # USI engine
        try:
            if self.stm == BLACK:                
                self.usib.command('stop\n')        
            else:                
                self.usiw.command('stop\n')            
        except:
            pass


    def time_control(self, b):
        dialog = gtk.Dialog("Time Control", None, 0, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))  
        dialog.set_title('Time Control')        

        fr = gtk.Frame("Available Time")
        vb = gtk.VBox(False, 0)
        fr.add(vb)
        dialog.vbox.pack_start(fr, False, True, 0)
        dialog.vbox.set_spacing(20)

        # available time - hours
        minimum = 0
        maximum = 10
        default = self.tc_hours        
        adj_hours = gtk.Adjustment(float(default), float(minimum), float(maximum), 1, 5, 0)               
        spinner = gtk.SpinButton(adj_hours, 1.0, 0)        
        al = gtk.Alignment(xalign=1.0, yalign=0.0, xscale=0.0, yscale=0.0)
        al.add(spinner)
        hb = gtk.HBox(False, 0)
        hb.pack_start(gtk.Label("Hours:"), False, False, 0)        
        hb.pack_start(al, True, True, 10)
        vb.pack_start(hb, False, True, 0)
        
        # available time - minutes
        minimum = 0
        maximum = 59
        default = self.tc_mins             
        adj_mins = gtk.Adjustment(float(default), float(minimum), float(maximum), 1, 5, 0)               
        spinner = gtk.SpinButton(adj_mins, 1.0, 0)        
        al = gtk.Alignment(xalign=1.0, yalign=0.0, xscale=0.0, yscale=0.0)
        al.add(spinner)        
        lbl = gtk.Label("Minutes :")
        hb = gtk.HBox(False, 0)
        hb.pack_start(lbl, False, False, 0)
        hb.pack_start(al, True, True, 10)
        vb.pack_start(hb, False, True, 0)       

        # byoyomi - seconds
        fr = gtk.Frame("Byoyomi")
        dialog.vbox.pack_start(fr, False, True, 0)
        minimum = 0
        maximum = 60
        default = self.tc_byoyomi        
        adj_secs = gtk.Adjustment(float(default), float(minimum), float(maximum), 1, 5, 0)               
        spinner = gtk.SpinButton(adj_secs, 1.0, 0)        
        al = gtk.Alignment(xalign=1.0, yalign=0.0, xscale=0.0, yscale=0.0)
        al.add(spinner)
        hb = gtk.HBox(False, 0)
        hb.pack_start(gtk.Label("Seconds:"), False, False, 0)        
        hb.pack_start(al, True, True, 10)
        fr.add(hb)        

        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.show_all()
                
        while True:
            response = dialog.run()
            if response != gtk.RESPONSE_OK:
                # cancelled - exit loop
                break
            # OK pressed - validate input"
            hours = int(adj_hours.value)
            minutes = int(adj_mins.value)
            seconds = int(adj_secs.value)            
            if hours == 0 and minutes ==0 and seconds == 0:                
                self.gui.info_box("Time fields cannot all be zero!")
            else:
                # input ok - exit loop
                self.tc_hours = hours
                self.tc_mins = minutes
                self.tc_byoyomi = seconds       

                time_ms = (self.tc_hours * 60 * 60 + self.tc_mins * 60) * 1000
                # btime, wtime ,byoyomi times in milliseconds
                self.time_left = [time_ms, time_ms, self.tc_byoyomi * 1000]

                self.gui.set_toolbar_time_control(self.time_left, 0, WHITE)
                self.gui.set_toolbar_time_control(self.time_left, 0, BLACK)                          

                break                  
       
        dialog.destroy()


    def set_players(self, b):
        dialog = gtk.Dialog("Players", None, 0, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK)) 
        dialog.set_title('Set Players')
    
        elist = self.engine_manager.get_engine_list()       
        
        # White Player 
        fr = gtk.Frame("White")
        dialog.vbox.pack_start(fr, True, True, 15)
        #dialog.vbox.set_spacing(20)
        vb = gtk.VBox(False, 0)
        fr.add(vb)
               
        comboboxw = gtk.combo_box_new_text()        
        comboboxw.append_text("Human")
        
        if self.player[WHITE] == "Human":
            comboboxw.set_active(0)

        i = 1
        for (engine_name, path) in elist:            
            comboboxw.append_text(engine_name)
            
            if engine_name == self.player[WHITE]:
                comboboxw.set_active(i)
            i += 1        
        vb.pack_start(comboboxw, True, True, 15)       


        # Black Player 
        fr = gtk.Frame("Black")
        dialog.vbox.pack_start(fr, True, True, 15)
        vb = gtk.VBox(False, 0)
        fr.add(vb)
                       
        comboboxb = gtk.combo_box_new_text()
        comboboxb.append_text("Human")
        if self.player[BLACK] == "Human":
            comboboxb.set_active(0)

        i = 1
        for (engine_name, path) in elist:            
            comboboxb.append_text(engine_name)            
            if engine_name == self.player[BLACK]:
                comboboxb.set_active(i)                
            i += 1        
        vb.pack_start(comboboxb, True, True, 15)        
        
        dialog.show_all()        


        # If user hasn't clicked on OK then exit now
        if dialog.run() != gtk.RESPONSE_OK:
            dialog.destroy()
            return                
        
        
        self.player[BLACK] = comboboxb.get_active_text()
        self.player[WHITE] = comboboxw.get_active_text()

        self.usib.set_engine(self.player[BLACK], None)
        self.usiw.set_engine(self.player[WHITE], None)

        self.gui.update_toolbar(self.player)
        
        dialog.destroy()               


    def set_level(self, b):        
        
        dialog = gtk.MessageDialog(
            None,  
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,  
            gtk.MESSAGE_QUESTION,  
            gtk.BUTTONS_OK_CANCEL,  
            None)
        dialog.set_title('Configure Engine')    
        markup = "<b>gShogi</b>"
        dialog.set_markup(markup)       

        adj = gtk.Adjustment(float(self.search_depth), float(0), float(39), 1, 5, 0)               
        spinner = gtk.SpinButton(adj, 1.0, 0)        
        al = gtk.Alignment(xalign=1.0, yalign=0.0, xscale=0.0, yscale=0.0)
        al.add(spinner)
        al.show()
        spinner.show()        

        tbl = gtk.Table(1, 2, True)
        tbl.attach(gtk.Label("Search Depth:"), 0, 1, 0, 1)
        tbl.attach(al, 1, 2, 0, 1)       

        #some secondary text        
        markup = 'Configure Options'        
       
        dialog.format_secondary_markup(markup)      
        
        dialog.vbox.add(tbl)       

        dialog.show_all()        

        # If user hasn't clicked on OK then exit now
        if dialog.run() != gtk.RESPONSE_OK:
            dialog.destroy()
            return

        # user clicked OK so update with the values entered        
        depth = adj.get_value()        
        dialog.destroy()               
        
        # set search depth if valid
        try:
            idepth = int(depth)
            if (idepth >= 0 and idepth <= 39):
                self.search_depth = idepth
                engine.depth(self.search_depth)
        except ValueError:
                pass    


# class to save settings on program exit and restore on program start
class Settings:
    pass


def run():
    Game()        
    #gobject.threads_init()
    gtk.gdk.threads_init() 
    gtk.gdk.threads_enter()   
    gtk.main()
    gtk.gdk.threads_leave()
    return 0     

if __name__ == "__main__":        
    run()
