#
#   gui.py
#
#   This file is part of gshogi   
#
#   gshogi is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   gshogi is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with gshogi.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk, os
import gobject, time
from constants import *

class Gui:  

    def build_gui(self): 

        self.zonebg_colour = '#C5B358'
        self.boardbg_colour = '#645452'      

        self.gobactive = False  

         # Create Main Window
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)               
        self.window.set_title(NAME + " " + VERSION)        
        self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))

        # Set a handler for delete_event that immediately exits GTK.
        self.window.connect("delete_event", self.game.delete_event) 

        # connect to expose event so pieces can be resized when user resizes the window      
        self.window.connect("expose_event", self.expose_event)        

        main_vbox = gtk.VBox(False, 25)                
        self.window.add(main_vbox)
        main_vbox.show()

        # menu
        # Create a UIManager instance
        uimanager = gtk.UIManager()

        # Add the accelerator group to the toplevel window
        accelgroup = uimanager.get_accel_group()
        self.window.add_accel_group(accelgroup)

        # main action group
        actiongroup = gtk.ActionGroup('UIManagerAG')        
        self.actiongroup = actiongroup

        # action group for toggleactions
        ta_action_group = gtk.ActionGroup('AGPromoteMode')     
        ta_action_group.add_toggle_actions([('promotemode', None, '_Ask Before Promoting', None, None,
                                               self.promote_mode)])
        ta_action_group.add_toggle_actions([('pieceset', None, '_Use Western Pieces', None, None,
                                               self.use_western_pieces)])
        self.ta_action_group = ta_action_group

        # Create actions
        actiongroup.add_actions([('NewGame', gtk.STOCK_NEW, '_New Game', None, 'New Game', self.game.new_game),
                                 ('Quit', gtk.STOCK_QUIT, '_Quit', None, 'Quit the Program', self.game.quit_game),                                                                 
                                 ('LoadGame', gtk.STOCK_OPEN, '_Load Game', None, 'Load Game', self.game.load_game),
                                 ('SaveGame', gtk.STOCK_SAVE, '_Save Game', None, 'Save Game', self.game.save_game),                                 
                                 ('File', None, '_File'),                                             
                                 ('Edit', None, '_Edit'),                                                             
                                 ('Undo', gtk.STOCK_UNDO, '_Undo Move', "<Control>U", 'Undo Move', self.game.undo_move),
                                 ('Redo', gtk.STOCK_REDO, '_Redo Move', "<Control>R", 'Redo Move', self.game.redo_move), 
                                 ('MoveNow', None, '_Move Now', "<Control>M", 'Move Now', self.game.move_now), 
                                 ('TimeControl', None, '_Time Control', None, 'Time Control', self.game.time_control),
                                 # ConfigureEngine1 - this name is used in engine_manager. Don't change it.
                                 ('ConfigureEngine1', None, '_Configure Engine 1', None, 'Configure Engine 1', self.engine_manager.configure_engine),
                                 ('ConfigureEngine2', None, '_Configure Engine 2', None, 'Configure Engine 2', self.engine_manager.configure_engine),
                                 ('Players', None, '_Players', None, 'Players', self.game.set_players), 
                                 ('Engines', None, '_Engines', None, 'Engines', self.engine_manager.engines), 
                                 ('Options', None, '_Options'),                                                             
                                 ('About', gtk.STOCK_ABOUT, '_About', None, 'Show About Box', self.about_box),                                 
                                 ('Help', None, '_Help'),
                                ])        

        # Add the actiongroups to the uimanager
        uimanager.insert_action_group(actiongroup, 0)
        uimanager.insert_action_group(ta_action_group, 1)

        ui = '''<ui>
        <menubar name="MenuBar">
            <menu action="File">        
                <menuitem action="NewGame"/>
                <separator/>          
                <menuitem action="LoadGame"/>
                <menuitem action="SaveGame"/>  
                <separator/>
                <menuitem action="Quit"/>                        
            </menu>
            <menu action="Options">                 
                <menuitem action="Undo"/>
                <menuitem action="Redo"/> 
                <menuitem action="MoveNow"/> 
                <separator/>  
                <menuitem action="promotemode"/> 
                <menuitem action="pieceset"/> 
                <separator/>
                <menuitem action="TimeControl"/>
                <menuitem action="ConfigureEngine1"/> 
                <menuitem action="ConfigureEngine2"/> 
                <menuitem action="Players"/>                 
                <menuitem action="Engines"/> 
            </menu>
            <menu action="Help">                
                <menuitem action="About"/> 
            </menu>
        </menubar>
        </ui>'''

        # Add a UI description
        uimanager.add_ui_from_string(ui)

        vbox2 = gtk.VBox(False, 0)
        main_vbox.pack_start(vbox2, False)
  
        # Create a MenuBar
        menubar = uimanager.get_widget('/MenuBar')                
        vbox2.pack_start(menubar, False)

        # create a toolbar
        toolbar = gtk.Toolbar()
        vbox2.pack_start(toolbar, False)         

        # populate toolbar
        toolitem = gtk.ToolItem()        

        # 2 rows, 4 columns, not homogeneous
        tb = gtk.Table(2,5, False)
        
        # gtk.SHADOW_NONE, SHADOW_IN, SHADOW_OUT, SHADOW_ETCHED_IN, SHADOW_ETCHED_OUT

        self.side_to_move = [gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_ETCHED_IN), gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_ETCHED_IN)]
        self.side_to_move[BLACK].set_alignment(0.5, 0.4)
        self.side_to_move[WHITE].set_alignment(0.5, 0.4)        
        
        tb.attach(self.side_to_move[WHITE], 0, 1, 0, 1) 
        tb.attach(self.side_to_move[BLACK], 0, 1, 1, 2)    

        lw = gtk.Label("White: ")
        lb = gtk.Label("Black: ")
        lw.set_alignment(0, 0.5)
        lb.set_alignment(0, 0.5)    
        tb.attach(lw, 1, 2, 0, 1) 
        tb.attach(lb, 1, 2, 1, 2)

        self.engines_lblw = gtk.Label("gshogi")        
        self.engines_lblw.set_use_markup(True)
        self.engines_lblw.set_alignment(0, 0.5)

        self.engines_lblb = gtk.Label("human")        
        self.engines_lblb.set_use_markup(True)
        self.engines_lblb.set_alignment(0, 0.5)        

        tb.attach(self.engines_lblw, 2, 3, 0, 1) 
        tb.attach(self.engines_lblb, 2, 3, 1, 2)

        # time control    
        self.tc_lbl = [gtk.Label("00:45:00 00/10"), gtk.Label("00:45:00 00/10")] 
        self.tc_lbl[BLACK].set_alignment(0, 0.5)
        self.tc_lbl[WHITE].set_alignment(0, 0.5)      
    
        tb.attach(self.tc_lbl[WHITE], 3, 4, 0, 1) 
        tb.attach(self.tc_lbl[BLACK], 3, 4, 1, 2)

        toolitem.add(tb)
        toolbar.insert(toolitem, -1)

        # add a vertical separator
        hb = gtk.HBox(False, 0)
        toolitem = gtk.ToolItem()
        vsep = gtk.VSeparator()
        hb.pack_start(vsep, True, True, 10)
        toolitem.add(hb)
        toolbar.insert(toolitem, -1)

        # stop/go buttons
        hb = gtk.HBox(False, 0)
        self.gobutton = gtk.ToolButton(gtk.STOCK_YES)
        
        self.gobutton.connect('clicked', self.game.go_clicked)
        
        self.stopbutton = gtk.ToolButton(gtk.STOCK_NO)        
        self.stopbutton.connect('clicked', self.game.stop_clicked)        

        # set_tooltip_text needs PyGTK 2.12 and above.
        try:
            self.gobutton.set_tooltip_text('go')
            self.stopbutton.set_tooltip_text('stop')
        except AttributeError, ae:
            pass

        hb.pack_start(self.stopbutton, False)
        hb.pack_start(self.gobutton, False)        
        
        toolitem = gtk.ToolItem()
        toolitem.add(hb)
        toolbar.insert(toolitem, -1)

        # add a vertical separator
        hb = gtk.HBox(False, 0)
        toolitem = gtk.ToolItem()
        vsep = gtk.VSeparator()
        hb.pack_start(vsep, True, True, 10)
        toolitem.add(hb)
        toolbar.insert(toolitem, -1)
        

        ###        
        main_hbox = gtk.HBox(False, 25)        
        
        # Create a 9x1 table for pieces captured by the white side
        self.setup_white_komadai(main_hbox)

        # Create a 9x9 table for the main board
        self.table = gtk.Table(9, 9, True)
        self.table.set_border_width(1)
        eb2 = gtk.EventBox()        
        eb2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black")) 
        eb2.add(self.table)       
        eb = gtk.EventBox()        
        eb.add(eb2)         
        eb.show()
        eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))       

        aspect_frame = gtk.AspectFrame(label=None, xalign=0.5, yalign=0.5, ratio=1.0, obey_child=False)
        aspect_frame.add(eb)
        main_hbox.pack_start(aspect_frame, True, True, 0)        

        # Create a 9x1 table for pieces captured by the black side
        self.setup_black_komadai(main_hbox)
       
        aspect_frame = gtk.AspectFrame(label=None, xalign=0.5, yalign=0.5, ratio=1.351, obey_child=False)
        aspect_frame.set_shadow_type(gtk.SHADOW_NONE)        
        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.7, yscale=0.7)
        al.add(main_hbox)
        aspect_frame.add(al)
        aspect_frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))        
        aspect_frame.set_border_width(0)
        self.af = aspect_frame
        main_vbox.pack_start(aspect_frame, True, True, 0)       

        # status bar 
        self.status_bar = gtk.Statusbar() 
        main_vbox.pack_start(self.status_bar, False, False, 0)        
        self.context_id = self.status_bar.get_context_id("gshogi statusbar") 

        self.actiongroup.get_action('MoveNow').set_sensitive(False)                

        self.window.show_all() 
        self.side_to_move[WHITE].hide()
        self.gobutton.set_sensitive(False)
        self.stopbutton.set_sensitive(False)       

        self.window.set_geometry_hints(self.window, min_width=378, min_height=378, max_width=-1, max_height=-1, base_width=-1, base_height=-1, width_inc=-1, height_inc=-1, min_aspect=-1.0, max_aspect=-1.0) 


    def set_refs(self, game, board, engine_manager):
        self.game = game
        self.board = board         
        self.engine_manager = engine_manager


    def init_board_square(self, image, x, y):
        event_box = gtk.EventBox()
        event_box.add(image) 
        self.table.attach(event_box, x, x+1, y, y+1)
        event_box.show()
        event_box.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#EBDFB0"))
        event_box.set_border_width(1)              
        event_box.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        data = (x, y)
        event_box.connect('button_press_event', self.game.square_clicked, data)
        image.show()  
        self.eb = event_box 

    #
    # set up the komadai for white
    # this is a column on the left of the board to hold pieces captured by white
    #
    def setup_white_komadai(self, hbox):       
        self.wcaptable = gtk.Table(9, 1, False)
        eb = gtk.EventBox()   
        eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))            
        eb.add(self.wcaptable)         
        eb.show()              

        al = gtk.Alignment(xalign=0.0, yalign=0.5, xscale=0.0, yscale=0.0)        
        al.add(eb)        
        hbox.pack_start(al, True, True, 0)       
        
        self.wcaptable.show()        
        

    #
    # set up the komadai for black
    # this is a column on the right of the board to hold pieces captured by black
    #   
    def setup_black_komadai(self, hbox): 
        self.bcaptable = gtk.Table(9, 1, False)       
        eb = gtk.EventBox()       
        eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour)) 
        eb.add(self.bcaptable)
        eb.show()        
        
        al = gtk.Alignment(xalign=1.0, yalign=0.5, xscale=0.0, yscale=0.0)
        al.add(eb)
        hbox.pack_start(al, True, True, 0)       
        self.bcaptable.show()


    def init_wcap_square(self, image, y, label):        

        hb = gtk.HBox(False, 0)
        hb.show()

        label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#FFFDD0"))        
        
        event_box2 = gtk.EventBox()        
        event_box2.add(label)        
        label.show()       
        hb.pack_start(event_box2, True, True, 0) 
        event_box2.show()
        event_box2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))        
        event_box2.set_border_width(1)        

        event_box = gtk.EventBox()
        event_box.add(image)        

        aspect_frame = gtk.AspectFrame(label=None, xalign=0.5, yalign=0.5, ratio=1.0, obey_child=False)
        aspect_frame.add(event_box)
        aspect_frame.show()        
        aspect_frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#002E63"))
        
        # possible shadow values
        # gtk.SHADOW_NONE, gtk.SHADOW_IN, gtk.SHADOW_OUT, gtk.SHADOW_ETCHED_IN, gtk.SHADOW_ETCHED_OUT
        aspect_frame.set_shadow_type(gtk.SHADOW_ETCHED_IN)          
        
        hb.pack_start(aspect_frame, True, True, 0) 
        x = 0      
        self.wcaptable.attach(hb, x, x+1, y, y+1) 
         
        hb.show()        
   
        event_box.show()        
        event_box.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.zonebg_colour))
        event_box.set_border_width(0) 

        event_box.add_events(gtk.gdk.BUTTON_PRESS_MASK)       
        data = (x, y, WHITE)
        event_box.connect('button_press_event', self.game.cap_square_clicked, data)        
        image.show()        
        

    def init_bcap_square(self, image, y, label):

        hb = gtk.HBox(False, 0)
        hb.show() 

        label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#FFFDD0"))        
        
        event_box2 = gtk.EventBox()        
        event_box2.add(label)        
        label.show()       
        hb.pack_end(event_box2, True, True, 0) 
        event_box2.show()
        event_box2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.boardbg_colour))        
        event_box2.set_border_width(1)
        
        event_box = gtk.EventBox()
        event_box.add(image) 

        aspect_frame = gtk.AspectFrame(label=None, xalign=0.5, yalign=0.5, ratio=1.0, obey_child=False)
        aspect_frame.add(event_box)
        aspect_frame.show()
        aspect_frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#002E63"))   
        aspect_frame.set_shadow_type(gtk.SHADOW_ETCHED_IN) 
        hb.pack_end(aspect_frame, True, True, 0) 

        x = 0      
        self.bcaptable.attach(hb, x, x+1, y, y+1) 
         
        hb.show()        
   
        event_box.show()       
        event_box.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.zonebg_colour))
        event_box.set_border_width(0)        

        event_box.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        data = (x, y, BLACK)
        event_box.connect('button_press_event', self.game.cap_square_clicked, data)
       
        image.show()

        
    # return a rect of the size of a square on the table
    def get_square_size(self):       
        return self.eb.allocation


    # about box
    def about_box(self, widget):
        about = gtk.AboutDialog()        
        about.set_program_name(NAME)
        about.set_version(VERSION)
        about.set_copyright(u'Copyright \u00A9 2010 John Cheetham')                             
        about.set_comments("gshogi is a a program to play shogi (Japanese Chess).")
        about.set_authors(["John Cheetham"])
        about.set_website("http://www.johncheetham.com/projects/gshogi/index.shtml")
        about.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(self.game.prefix, "images/logo.png")))

        license = '''gshogi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

gshogi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gshogi.  If not, see <http://www.gnu.org/licenses/>.'''

        about.set_license(license)
        about.run()
        about.destroy()


    def set_status_bar_msg(self, msg):
        self.status_bar.push(self.context_id, msg)

    # ask before promoting
    def promote_mode(self, w):                  
        if w.get_active():        
            self.game.set_promotion_mode(True)            
        else:           
            self.game.set_promotion_mode(False)      
    
     
    def get_western(self):    
        return self.ta_action_group.get_action('pieceset').get_active()


    def set_western(self):    
        self.ta_action_group.get_action('pieceset').set_active(True)


    def use_western_pieces(self, w):                  
        if w.get_active():        
            self.board.use_western_pieces(True)            
        else:   
            self.board.use_western_pieces(False)


    def expose_event(self, widget, event):            
        if not self.gobactive:            
            self.gobactive = True            
            gobject.idle_add(self.resize_pieces)            


    # user is resizing the window
    # call board.update to resize the pieces
    #
    # This must be called after the window has been realised
    # so it will get the latest correct window size.
    # Connecting to the window expose event achieves this
    #
    def resize_pieces(self):        
        self.board.refresh_screen()    
        self.gobactive = False
        return False

            
    # disable some functionality if computer is thinking    
    def disable_menu_items(self):        
        self.actiongroup.get_action('Undo').set_sensitive(False)
        self.actiongroup.get_action('Redo').set_sensitive(False)
        self.actiongroup.get_action('ConfigureEngine1').set_sensitive(False)
        self.actiongroup.get_action('ConfigureEngine2').set_sensitive(False)
        self.actiongroup.get_action('NewGame').set_sensitive(False)
        self.actiongroup.get_action('LoadGame').set_sensitive(False)
        self.actiongroup.get_action('SaveGame').set_sensitive(False) 
        self.actiongroup.get_action('Engines').set_sensitive(False)
        self.actiongroup.get_action('Players').set_sensitive(False)
        self.actiongroup.get_action('TimeControl').set_sensitive(False) 
        self.actiongroup.get_action('MoveNow').set_sensitive(True)
        self.gobutton.set_sensitive(False) 
        self.stopbutton.set_sensitive(True)      


    # enable some functionality when computer is finished thinking    
    def enable_menu_items(self):                
        self.actiongroup.get_action('Undo').set_sensitive(True)
        self.actiongroup.get_action('Redo').set_sensitive(True)
        self.actiongroup.get_action('ConfigureEngine1').set_sensitive(True)
        self.actiongroup.get_action('ConfigureEngine2').set_sensitive(True)
        self.actiongroup.get_action('NewGame').set_sensitive(True)
        self.actiongroup.get_action('LoadGame').set_sensitive(True)
        self.actiongroup.get_action('SaveGame').set_sensitive(True)
        self.actiongroup.get_action('Engines').set_sensitive(True)
        self.actiongroup.get_action('Players').set_sensitive(True)
        self.actiongroup.get_action('TimeControl').set_sensitive(True)
        self.actiongroup.get_action('MoveNow').set_sensitive(False)         
        self.gobutton.set_sensitive(True)
        self.stopbutton.set_sensitive(False) 


    def enable_go_button(self):
        self.gobutton.set_sensitive(True)


    def disable_go_button(self):
        self.gobutton.set_sensitive(False)


    def disable_stop_button(self):
        self.stopbutton.set_sensitive(False)


    def enable_stop_button(self):
        self.stopbutton.set_sensitive(True)


    def promote_popup(self):   
        
        dialog = gtk.Dialog("Promotion",
            None,  
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,              
            ("Yes", gtk.RESPONSE_YES,
             "No", gtk.RESPONSE_NO,
             "Cancel", gtk.RESPONSE_CANCEL))         
       
        dialog.vbox.pack_start(gtk.Label('\nPromote piece?\n'))

        dialog.show_all()  
        
        response = dialog.run()        
        dialog.destroy()
        
        return response
  

    def update_toolbar(self, player):                    
        self.engines_lblw.set_markup('<b>' + player[WHITE] + ' </b>')
        self.engines_lblb.set_markup('<b>' + player[BLACK] + ' </b>')           


    #
    # Update the clocks on the display
    # These countdown the time while the player is thinking    
    #
    def set_toolbar_time_control(self, time_left, move_time, side_to_move):       

        # subtract the move time from the players available time
        new_time = (time_left[side_to_move]) - move_time        
    
        byoyomi = time_left[2]

        # If no available time use byoyomi time
        if new_time <= 0:
            hours = 0
            mins = 0
            secs = 0
            byo = byoyomi - move_time
            if byo < 0:
                byo = 0
        else:            
            secs = int(new_time / 1000)            
            hours = int(secs / 3600)            
            secs = secs - (hours * 3600)
            mins = int(secs / 60)                
            secs = int(secs - (mins * 60))            
            byo = 0            
   
        byoyomi = int(byoyomi / 1000)
        byo = int(byo / 1000)

        shours = str(hours)
        if len(shours) < 2:
            shours = '0' + shours

        smins = str(mins)
        if len(smins) < 2:
            smins = '0' + smins

        ssecs = str(secs)
        if len(ssecs) < 2:
            ssecs = '0' + ssecs

        sbyo = str(byo)
        if len(sbyo) < 2:
            sbyo = '0' + sbyo
        sbyoyomi = str(byoyomi)
        if len(sbyoyomi) < 2:
            sbyoyomi = '0' + sbyoyomi       

        self.tc_lbl[side_to_move].set_text(shours + ':' + smins + ':' + ssecs + '  ' + sbyo + '/' + sbyoyomi)


    def info_box(self, msg):
        dialog = gtk.MessageDialog(
            None,  
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,  
            gtk.MESSAGE_INFO,  
            gtk.BUTTONS_OK,  
            None)

        #markup = "<b>" + msg + "</b>"
        markup = msg
        dialog.set_markup(markup)
        dialog.set_title('Info')      

        #some secondary text
        markup = msg                
       
        #dialog.format_secondary_markup(markup)      

        dialog.show_all()
        dialog.run()
        dialog.destroy()


    def set_side_to_move(self, stm):
        self.side_to_move[stm].show()
        self.side_to_move[stm ^ 1].hide()   


    # set the base window size at startup
    def set_window_size(self): 
        screen = self.window.get_screen()        
        screen_width = screen.get_width()  
        screen_height = screen.get_height()        
        
        if screen_width > 918 and screen_height > 724:            
            w = 918
            h = 724                       
        elif screen_width > 658 and screen_height > 523: 
            w = 658
            h = 523
        else:
            w = 511
            h = 406        
        
        self.window.resize(w, h)



