#
#   pieces.py
#
#   This file is part of gshogi   
#
#   gshogi is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   gshogi is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with gshogi.  If not, see <http://www.gnu.org/licenses/>.
#   
# The board pieces are borrowed from xshogi (part of gnushogi)
# which borrowed them from CRANES Shogi. The following relates to the 
# board pieces.
#
# Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts.
# Enhancements Copyright 1992 Free Software Foundation, Inc.
# Enhancements for XShogi Copyright 1993, 1994, 1995 Matthias Mutz
# Copyright (c) 1999 Michael Vanier and the Free Software Foundation
#
#
# The following terms apply to Digital Equipment Corporation's copyright
# interest in XBoard:
# ------------------------------------------------------------------------
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Digital not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
# DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
# ------------------------------------------------------------------------

import gtk
import os
import traceback

class Pieces:
    

    def __init__(self):
        self.western = False 
        self.scale_factor = 1.0


    def load_pieces(self, prefix):
        
        #  0 - unoccupied        
        #  1 - black pawn        
        #  2 - black lance        
        #  3 - black knight        
        #  4 - black silver general       
        #  5 - black gold general        
        #  6 - black bishop       
        #  7 - black rook       
        #  8 - black king
        #  9 - promoted black pawn       
        # 10 - promoted black lance        
        # 11 - promoted black knight       
        # 12 - promoted black silver general        
        # 13 - promoted black bishop       
        # 14 - promoted black rook
        # 15 - white pawn        
        # 16 - white lance        
        # 17 - white knight       
        # 18 - white silver general       
        # 19 - white gold general       
        # 20 - white bishop       
        # 21 - white rook        
        # 22 - white king        
        # 23 - promoted white pawn        
        # 24 - promoted white lance       
        # 25 - promoted white knight       
        # 26 - promoted white silver general       
        # 27 - promoted white bishop       
        # 28 - promoted white rook        
        
        self.piece_pixbuf = self.load_pixbufs('eastern', prefix)
        self.western_piece_pixbuf = self.load_pixbufs('western', prefix)               


    def load_pixbufs(self, piece_set, prefix):
        images = ['none', \
                  'pawnB', 'lanceB', 'knightB', 'silverB', 'goldB', \
                  'bishopB', 'rookB', 'kingB', 'pawnPB', 'lancePB', \
                  'knightPB', 'silverPB', 'bishopPB', 'rookPB',     \
                  'pawnW', 'lanceW', 'knightW', 'silverW', 'goldW', \
                  'bishopW', 'rookW', 'kingW', 'pawnPW', 'lancePW', \
                  'knightPW', 'silverPW', 'bishopPW', 'rookPW'      \
                 ]        
        piece_pixbuf = []
        for image in images:
            image = 'images/' + piece_set + '/' + image + '.png'            
            piece_pixbuf.append(gtk.gdk.pixbuf_new_from_file(os.path.join(prefix, image)))

        return piece_pixbuf


    def getpixbuf(self, piece):
        # pieces contains the list of possible pieces
        pieces = [" -", " p", " l", " n", " s", " g", " b", " r", " k", "+p", "+l", "+n", "+s", "+b", "+r", " P", " L", " N", " S", " G", " B", " R", " K", "+P", "+L", "+N", "+S", "+B", "+R"]         

        try:
            idx = pieces.index(piece) 
        except ValueError, ve:
            traceback.print_exc()
            print "error piece not found, piece =", piece 

        if (self.western):
            pixbuf = self.western_piece_pixbuf[idx]            
        else:
            pixbuf = self.piece_pixbuf[idx]

        width = int(self.scale_factor * 64)
        height = int(self.scale_factor * 64)
        spb = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_HYPER)        
        return spb                


    def use_western_pieces(self, wps):
        if (wps):
            self.western = True
        else:
            self.western = False    


    def set_scale_factor(self, factor):        
        if factor < 0:                      
            return     
        elif factor < 0.3:
            factor = 0.3            
        elif factor > 3.0:
            factor = 3.0 
        
        self.scale_factor = factor * 0.95            
        
        


